/*
 * ***************************************************************************************************
 *
 * 		Copyright (C) 2014-2019 Gowin Semiconductor Technology Co.,Ltd.
 * 		
 * @file        GOWIN_M1_sdcard.c
 * @author      Embedded Development Team
 * @version	    V1.0.0
 * @date        2019-1-1 09:00:00
 * @brief       This file contains all the functions prototypes for the SD-Card firmware library.
 ****************************************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "GOWIN_M1_sdcard.h"

int sd_sector_read(uint8_t *out, uint32_t sector)
{
	int i;
	uint32_t *p = (uint32_t *)out;

	if (0 == SDCARD->INITIALISED)
	{
		return -1;
	}

	SDCARD->BASEADDR = sector;
	SDCARD->READEN = 1;
	while (SDCARD->READEN == 1);

	for (i = 0; i < 128; i++)
	{
		*p++ = SDCARD->TRANSDATA[i];
	}

	return 0;
}

int sdcard_read_package(unsigned char *data, unsigned int base_addr)
{
	int flag_readen=1;
	int i = 0;
	int j = 0;
	unsigned int buff = 0;
	
	if(SDCARD->INITIALISED == 0)
	{
		return -1;
	}

	SDCARD->BASEADDR = base_addr;
	SDCARD->READEN = 1;
	
	while(flag_readen)
	{
		flag_readen = SDCARD->READEN;
	}

	for(i = 0; i < 128 && j < SDCARD_PACKAGE_SIZE; i++)
	{
		buff = SDCARD->TRANSDATA[i];
		/*
		data[j++] = (char)((buff & 0x00ff0000) >> 16);
		data[j++] = (char)((buff & 0xff000000) >> 24);
		data[j++] = (char)( buff & 0x000000ff);
		data[j++] = (char)((buff & 0x0000ff00) >>  8);
		*/
		
		data[j++] = (char)((buff & 0x0000ff00) >>  8);
		data[j++] = (char)( buff & 0x000000ff);
		data[j++] = (char)((buff & 0xff000000) >> 24);
		data[j++] = (char)((buff & 0x00ff0000) >> 16);
	}

	return 0;
}

int SD_ReadDisk(unsigned char* buf, unsigned int sector, unsigned int cnt)
{
	unsigned char* dst_buf = buf;
	unsigned int   src_addr = sector;
	int i = 0;

	/* Disable the machine & timer interrupts */
	//clear_csr(NDS_MIE, MIP_MEIP);
	//clear_csr(NDS_MIE, MIP_MTIP);

	for(i = 0; i < cnt; i++)
	{
		if(0 == sdcard_read_package(dst_buf, src_addr))
		{
			sector  += 512;
			dst_buf += 512;
		}
		else
		{
			return -1;
		}
	}

	/* Enable the Machine-External bit in MIE */
	//set_csr(NDS_MIE, MIP_MEIP);
	/* Enable the Machine-Timer bit in MIE */
	//set_csr(NDS_MIE, MIP_MTIP);

	return 0;
}
