/****************************************Copyright (c)****************************************************
**
**                                 www.gowinsemi.com
**
**--------------File Info---------------------------------------------------------------------------------
** File name:               main.c
** Last modified Date:      2019-06-12
** Last Version:            1.0.0
** Descriptions:
**
**--------------------------------------------------------------------------------------------------------
** Created by:              feiyun.wu
** Created date:            2019-06-12
** Version:                 1.0.0
** Descriptions:            .c
**
**--------------------------------------------------------------------------------------------------------
** Modified by:
** Modified date:
** Version:
** Descriptions:
**
*********************************************************************************************************/
#include <stdio.h>

#include "platform.h"

#include "FreeRTOS.h"
#include "task.h"

#include "Driver_GPIO.h"


#define SW_STR_NAME							"freertos_riscv_2a18"
#define SW_STR_EDITION						"V1.0.0.190612a"
#define SW_STR_AUTHOR						"GW"


#define LOGO_PRINT_ON

#ifdef LOGO_PRINT_ON
#include "logo.h"
#endif	//LOGO_PRINT_ON


#define LED0_PIN				0
#define LED1_PIN				1

extern NDS_DRIVER_GPIO Driver_GPIO;


#define TASK0_TASK_PRIO			1
#define TASK0_STK_SIZE 			48
TaskHandle_t TASK0_Handler;
static void task0(void *pvParameters);

#define TASK1_TASK_PRIO			2
#define TASK1_STK_SIZE 			48
TaskHandle_t TASK1_Handler;
static void task1(void *pvParameters);


volatile int led0_flag = 0;
volatile int led1_flag = 0;


static void sw_edition_print(void)
{
	printf("\r\n");
	printf(starts_str);
	printf("Name:     "SW_STR_NAME"\r\n"
           "Edition:  "SW_STR_EDITION"\r\n"
		   "Compiled: "__DATE__", "__TIME__"\r\n"
			"Author:   "SW_STR_AUTHOR"\r\n");
	printf(starts_str);
	printf("\r\n");
}

static void prvSetupHardware( void )
{
	/* Ensure no interrupts execute while the scheduler is in an inconsistent
	state.  Interrupts are automatically enabled when the scheduler is
	started. */
	portDISABLE_INTERRUPTS();
}

void vAssertCalled(const char * pcFile, unsigned long ulLine)
{
	volatile unsigned long ul = 0;

	(void) pcFile;
	(void) ulLine;

	printf("ASSERT! Line %d, file %s\r\n", ( int )ulLine, pcFile);

	taskENTER_CRITICAL();
	{
		/* Set ul to a non-zero value using the debugger to step out of this
		function. */
		while (ul == 0)
		{
			portNOP();
		}
	}
	taskEXIT_CRITICAL();
}

static void gpio_init(void)
{
	Driver_GPIO.SetDir((1 << LED0_PIN) | (1 << LED1_PIN), NDS_GPIO_DIR_OUTPUT);
	Driver_GPIO.PinWrite(LED0_PIN, 1);
	Driver_GPIO.PinWrite(LED1_PIN, 1);
}

static void start_info_print(void)
{
	sw_edition_print();

#ifdef LOGO_PRINT_ON
	printf("\r\n");
	printf(starts_str);
	printf(logo_str);
	printf(starts_str);
	printf("\r\n");
#endif	//LOGO_PRINT_ON
}

static void misc_init(void)
{
	gpio_init();
	start_info_print();
}

static void task0(void *pvParameters)
{
    while (1)
	{
		if (0 == led0_flag)
		{
			Driver_GPIO.PinWrite(LED0_PIN, 0);
		}
		else
		{
			Driver_GPIO.PinWrite(LED0_PIN, 1);
		}

		led0_flag = !led0_flag;
		printf("0.task0\r\n");
		vTaskDelay(pdMS_TO_TICKS(250));
	}
}

static void task1(void *pvParameters)
{
    while (1)
	{
		if (0 == led1_flag)
		{
			Driver_GPIO.PinWrite(LED1_PIN, 0);
		}
		else
		{
			Driver_GPIO.PinWrite(LED1_PIN, 1);
		}

		led1_flag = !led1_flag;
		printf("1.task1\r\n\r\n");
		vTaskDelay(pdMS_TO_TICKS(1000));
	}
}

int main(void)
{
	prvSetupHardware();
	misc_init();

	taskENTER_CRITICAL();
	{
		xTaskCreate((TaskFunction_t )task0,
					(const char *   )"task0",
					(const uint16_t )TASK0_STK_SIZE,
					(void * const   )NULL,
					(UBaseType_t    )TASK0_TASK_PRIO,
					(TaskHandle_t *  )&TASK0_Handler);

		xTaskCreate((TaskFunction_t )task1,
					(const char *   )"task1",
					(const uint16_t )TASK1_STK_SIZE,
					(void * const   )NULL,
					(UBaseType_t    )TASK1_TASK_PRIO,
					(TaskHandle_t *  )&TASK1_Handler);
	}
    taskEXIT_CRITICAL();

	vTaskStartScheduler();

	while (1);

	return 0;
}

/*********************************************************************************************************
  END FILE
*********************************************************************************************************/
