/****************************************Copyright (c)****************************************************
**
**                                 www.gowinsemi.com
**
**--------------File Info---------------------------------------------------------------------------------
** File name:               main.c
** Last modified Date:      2019-06-12
** Last Version:            1.0.0
** Descriptions:
**
**--------------------------------------------------------------------------------------------------------
** Created by:              feiyun.wu
** Created date:            2019-06-12
** Version:                 1.0.0
** Descriptions:            .c
**
**--------------------------------------------------------------------------------------------------------
** Modified by:
** Modified date:
** Version:
** Descriptions:
**
*********************************************************************************************************/
#include <stdio.h>

#include "platform.h"
#include "includes.h"

#include "Driver_GPIO.h"


#define SW_STR_NAME							"ucosiii_riscv_2a18_exflash"
#define SW_STR_EDITION						"V1.0.0.190612a"
#define SW_STR_AUTHOR						"GW"


#define LOGO_PRINT_ON

#ifdef LOGO_PRINT_ON
#include "logo.h"
#endif	//LOGO_PRINT_ON


#define PLIC_SW_SOURCE			1

#define TICK_CLOCK_HZ			(MTIMERFREQ)
#define MTIME_BASE_ADDR			(0xE6000000)


#define LED0_PIN				0
#define LED1_PIN				1

extern NDS_DRIVER_GPIO Driver_GPIO;


#define STARTUP_TASK_PRIO		3
#define STARTUP_STK_SIZE 		80
OS_TCB startup_task_tcb;
CPU_STK STARTUP_TASK_STK[STARTUP_STK_SIZE];
static void startup_task(void *p_arg);

#define APP_TASK_PRIO			(STARTUP_TASK_PRIO + 1)
#define APP_STK_SIZE 			80
OS_TCB app_task_tcb;
CPU_STK APP_TASK_STK[APP_STK_SIZE];
static void app_task(void *p_arg);


volatile int led0_flag = 0;
volatile int led1_flag = 0;


extern void SysTick_Handler(void);
extern void Software_IRQHandler();


static void sw_edition_print(void)
{
	printf("\r\n");
	printf(starts_str);
	printf("Name:     "SW_STR_NAME"\r\n"
           "Edition:  "SW_STR_EDITION"\r\n"
		   "Compiled: "__DATE__", "__TIME__"\r\n"
			"Author:   "SW_STR_AUTHOR"\r\n");
	printf(starts_str);
	printf("\r\n");
}

static void gpio_init(void)
{
	Driver_GPIO.SetDir((1 << LED0_PIN) | (1 << LED1_PIN), NDS_GPIO_DIR_OUTPUT);
	Driver_GPIO.PinWrite(LED0_PIN, 1);
	Driver_GPIO.PinWrite(LED1_PIN, 1);
}

static void mtimer_next_time_set(void)
{
	volatile uint64_t *mtime = (volatile uint64_t * const)(MTIME_BASE_ADDR);
	volatile uint64_t *timecmp = (volatile uint64_t * const)(MTIME_BASE_ADDR + 8);

	*timecmp = *mtime + (TICK_CLOCK_HZ / OS_CFG_TICK_RATE_HZ);
}

void timer_swi_init(void)
{
	mtimer_next_time_set();
	__nds__plic_sw_enable_interrupt(PLIC_SW_SOURCE);
	set_csr(NDS_MIE, MIP_MTIP);
}

unsigned int plic_sw_claim_interrupt_func(void)
{
  unsigned int hart_id = __nds__mfsr(NDS_MHARTID);
  volatile unsigned int *claim_addr = (volatile unsigned int *)(NDS_PLIC_SW_BASE +
                                                                PLIC_CLAIM_OFFSET +
                                                                (hart_id << PLIC_CLAIM_SHIFT_PER_TARGET));
  return *claim_addr;
}

void plic_sw_complete_interrupt_func(void)
{
  unsigned int hart_id = __nds__mfsr(NDS_MHARTID);
  volatile unsigned int *claim_addr = (volatile unsigned int *)(NDS_PLIC_SW_BASE +
                                                                PLIC_CLAIM_OFFSET +
                                                                (hart_id << PLIC_CLAIM_SHIFT_PER_TARGET));
  *claim_addr = PLIC_SW_SOURCE;
}

/*void plic_sw_set_pending_func(void)
{
	//Set PLIC_SW source 1 to trigger Machine software interrupt
	//__nds__plic_sw_set_pending(1);

	volatile unsigned int *current_ptr = (volatile unsigned int *)(NDS_PLIC_SW_BASE +
                                                                 PLIC_PENDING_OFFSET +
                                                                 ((1 >> 5) << 2));
	*current_ptr = (1 << (1 & 0x1F));
}*/

void mtime_handler(void)
{
	clear_csr(NDS_MIE, MIP_MTIP);

	mtimer_next_time_set();
    SysTick_Handler();

    set_csr(NDS_MIE, MIP_MTIP);
}

void mswi_handler(void)
{
	//__nds__plic_sw_claim_interrupt();

	Software_IRQHandler();

	//__nds__plic_sw_complete_interrupt(1);
}

static void os_interrupt_init(void)
{
	__nds__plic_sw_set_priority(1, 1);
	__nds__plic_sw_disable_interrupt(1);

	clear_csr(NDS_MSTATUS, MSTATUS_MIE);
	clear_csr(NDS_MIE, MIP_MTIP);
}

static void start_info_print(void)
{
	sw_edition_print();

#ifdef LOGO_PRINT_ON
	printf("\r\n");
	printf(starts_str);
	printf(logo_str);
	printf(starts_str);
	printf("\r\n");
#endif	//LOGO_PRINT_ON
}

static void misc_init(void)
{
	gpio_init();
	start_info_print();
}

static void app_task(void *p_arg)
{
	OS_ERR os_err;

	(void)p_arg;

	while (1)
	{
		if (0 == led1_flag)
		{
			Driver_GPIO.PinWrite(LED1_PIN, 0);
		}
		else
		{
			Driver_GPIO.PinWrite(LED1_PIN, 1);
		}

		led1_flag = !led1_flag;
		printf("1.app_task\r\n\r\n");
		OSTimeDlyHMSM(0, 0, 0, 1000, OS_OPT_TIME_HMSM_STRICT, &os_err);
	}
}

static void startup_task(void *p_arg)
{
	OS_ERR os_err;
	CPU_SR_ALLOC();

	(void)p_arg;

	CPU_CRITICAL_ENTER();
	{
		OSTaskCreate((OS_TCB *		)&app_task_tcb,
					 (CPU_CHAR *	)"app_task",
					 (OS_TASK_PTR	)app_task,
					 (void *		)0,
					 (OS_PRIO		)APP_TASK_PRIO,
					 (CPU_STK *		)&APP_TASK_STK[0],
					 (CPU_STK_SIZE	)(APP_STK_SIZE / 10),
					 (CPU_STK_SIZE	)APP_STK_SIZE,
					 (OS_MSG_QTY	)0,
					 (OS_TICK		)0,
					 (void *		)0,
					 (OS_OPT		)(OS_OPT_TASK_STK_CHK | OS_OPT_TASK_STK_CLR),
					 (OS_ERR *		)&os_err);

		if (OS_ERR_NONE != os_err)
		{
			while (1);
		}
	}
	CPU_CRITICAL_EXIT();

    while (1)
	{
		if (0 == led0_flag)
		{
			Driver_GPIO.PinWrite(LED0_PIN, 0);
		}
		else
		{
			Driver_GPIO.PinWrite(LED0_PIN, 1);
		}

		led0_flag = !led0_flag;
		printf("0.startup_task\r\n");
		OSTimeDlyHMSM(0, 0, 0, 250, OS_OPT_TIME_HMSM_STRICT, &os_err);
	}
}

int main(void)
{
	OS_ERR os_err;
	CPU_SR_ALLOC();

	os_interrupt_init();
	misc_init();
	//Mem_Init();
    CPU_Init();

    OSInit(&os_err);
    if (OS_ERR_NONE != os_err)
	{
        while (1);
    }

    //App_OS_SetAllHooks();

	CPU_CRITICAL_ENTER();
	{
		OSTaskCreate((OS_TCB *		)&startup_task_tcb,
					 (CPU_CHAR *	)"startup_task",
					 (OS_TASK_PTR	)startup_task,
					 (void *		)0,
					 (OS_PRIO		)STARTUP_TASK_PRIO,
					 (CPU_STK *		)&STARTUP_TASK_STK[0],
					 (CPU_STK_SIZE	)(STARTUP_STK_SIZE / 10),
					 (CPU_STK_SIZE	)STARTUP_STK_SIZE,
					 (OS_MSG_QTY	)0,
					 (OS_TICK		)0,
					 (void *		)0,
					 (OS_OPT		)(OS_OPT_TASK_STK_CHK | OS_OPT_TASK_STK_CLR),
					 (OS_ERR *		)&os_err);

		if (OS_ERR_NONE != os_err)
		{
			while (1);
		}
	}
	CPU_CRITICAL_EXIT();

	OSStart(&os_err);
	while (1);

	return 0;
}

/*********************************************************************************************************
  END FILE
*********************************************************************************************************/
